from enum import Enum, unique
from typing import Final, FrozenSet

ROOT_NAMESPACE: Final[str] = "/"

DEFAULT_EMITTER_ID: Final[str] = "emitter"
DEFAULT_CHANNEL_PREFIX: Final[str] = "socket.io"

CHANNEL_SEPARATOR: Final[str] = "#"

RESERVED_EVENTS: FrozenSet[str] = frozenset(
    [
        "connect",
        "connect_error",
        "disconnect",
        "disconnecting",
        "newListener",
        "removeListener",
    ]
)


@unique
class PacketTypes(int, Enum):
    REGULAR = 2
    BINARY = 5


@unique
class RequestTypes(int, Enum):
    JOIN = 2
    LEAVE = 3
    DISCONNECT = 4



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['socketio_emitter']

package_data = \
{'': ['*']}

install_requires = \
['aioredis>=2.0.1,<3.0.0', 'msgpack>=1.0.3,<2.0.0', 'pydantic>=1.9.0,<2.0.0']

setup_kwargs = {
    'name': 'socket.io-redis-emitter',
    'version': '0.0.2.2',
    'description': 'An asynchronous Redis-based Socket.IO emitter for Python',
    'long_description': '# Socket.IO Redis Emitter\n\nThis is an asynchronous Redis-based [Socket.IO emitter](https://socket.io/docs/v4/emitting-events/) for Python.\n\n## Installation\n\n```bash\npip install socket.io-redis-emitter\n# or\npoetry add socket.io-redis-emitter\n```\n\n## Features\n\n- High quality, typed and modern Python codebase\n- Clean, concise and Pythonic API\n- Uses [aioredis](https://aioredis.readthedocs.io/en/latest/) as a Redis client\n- Supports namespaces, rooms and regular Socket.IO message emitting\n\n```python\nfrom aioredis import Redis\nfrom socketio_emitter import Emitter\n\nclient = Redis(...)\nemitter = Emitter(client=client)\n\nwith emitter.namespace("/nsp") as nsp:\n    with nsp.rooms("room1", "room2") as clients:\n        await clients.emit("machineStatus", {"status": "ok"})\n```\n\n- Remote requests to join, leave rooms or to disconnect \n\n```python\nfrom aioredis import Redis\nfrom socketio_emitter import Emitter\n\nclient = Redis(...)\nemitter = Emitter(client=client)\n\nwith emitter.namespace("/nsp") as nsp:\n    with nsp.rooms("room1", "room2") as clients:\n        await clients.join("room3")\n        # await clients.leave("room3")\n        # await clients.disconnect()\n```',
    'author': 'Roman Glushko',
    'author_email': 'roman.glushko.m@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/roma-glushko/socket.io-redis-emitter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Optional, Sequence, Tuple

import msgpack
from pydantic import BaseModel, Field

from socketio_emitter.consts import ROOT_NAMESPACE, PacketTypes, RequestTypes

EventName = str
MessageData = Tuple[EventName, Dict[str, Any]]


class MessageFlags(BaseModel):
    """
    Volatile:
        Sets a modifier for a subsequent event emission that the event data may be lost
        if the client is not ready to receive messages (because of network slowness or other issues,
        or because they’re connected through long polling
        and is in the middle of a request-response cycle).
    Compress:
        Sets the compress flag.
    """

    volatile: bool = False
    compress: bool = False


class MessageOptions(BaseModel):
    rooms: Optional[List[str]] = None
    flags: Optional[MessageFlags] = None
    except_rooms: Optional[List[str]] = Field(None, alias="except")

    class Config:
        allow_population_by_field_name = True


class Packet(BaseModel):
    data: MessageData
    type: PacketTypes = PacketTypes.REGULAR
    namespace: str = Field(ROOT_NAMESPACE, alias="nsp")

    class Config:
        allow_population_by_field_name = True


class Message(BaseModel):
    emitter_id: str
    packet: Packet
    options: MessageOptions

    def raw(self) -> bytes:
        """
        Generates a serialized version of a Socket.IO emit message
        Returns:
            Raw/Serialized Socket.IO emit message
        """
        return bytes(
            msgpack.packb(
                (
                    self.emitter_id,
                    self.packet.dict(by_alias=True),
                    self.options.dict(by_alias=True),
                )
            )
        )


class RequestOptions(BaseModel):
    rooms: Optional[Sequence[str]] = None
    except_rooms: Optional[Sequence[str]] = Field(None, alias="except")

    class Config:
        allow_population_by_field_name = True


class RoomRequest(BaseModel):
    type: RequestTypes
    rooms: Sequence[str]
    options: RequestOptions = Field(..., alias="opts")

    class Config:
        allow_population_by_field_name = True


class DisconnectRequest(BaseModel):
    type: RequestTypes = RequestTypes.DISCONNECT
    close: bool
    options: RequestOptions = Field(..., alias="opts")

    class Config:
        allow_population_by_field_name = True
